# Basic methods for plotting transcripts

.gs.init.plot.area = function( xlim, labels, cex.scale = par()$cex.axis, 
							cex.labels, fit.labels=TRUE, main="Gene Strip",
							xlab="Exons", ylab="Transcripts" ) {
	if( fit.labels ) {
		ht        = max( sapply( labels, strheight, units="inches" ) )
		ht.needed = ht * length( labels )
		ht.actual = par()$pin[2]
		if( missing( cex.labels ) ) {
			if( ht.needed > ht.actual ) {
				cex.labels = ht.actual / ht.needed
			}
			else {
				cex.labels = par()$cex.axis
			}
		}
		wd  = max( sapply( labels, strwidth, units="inches", cex=cex.labels ) )
		mai = par()$mai

		if( is.na( par()$tck ) ) {
			tck = -par()$tcl * par()$csi
		}
		else {
			tck = -par()$tck
		}
		ylab.height = strheight( ylab, "inches" )
		xlab.height = strheight( xlab, "inches" )

		lm     = wd + 6 * ylab.height
		bm     =      7 * xlab.height
		mai    = par()$mai
		mai[1] = bm
		mai[2] = lm

		par( mai = mai )
	}
	plot( 0, 0, type="n", xaxs="i", yaxt="n", bty="l", xlim=xlim, ylim=c( 0.5, length( labels ) + 0.5 ), cex.axis=cex.scale, xlab="", ylab="" )
	axis( 2, at=(seq_along( labels )), labels=as.character( labels ), tick=FALSE, las=2, cex.axis=cex.labels )
	title( outer=FALSE, main=main, xlab=NULL, ylab=NULL, line=1)
	title( outer=FALSE, main=NULL, xlab=xlab, ylab=NULL, line=2)
	title( outer=FALSE, main=NULL, xlab=NULL, ylab=ylab, line=floor( par()$mai[2] / par()$csi ) - 1 )
}

# data is a data.frame containing 7 fields;
# gene, transcript, exon, rect.x1, rect.y1, rect.x2 and rect.y2
.default.plotter = function( data ) {
	rect( data[ ,'rect.x1' ], data[ ,'rect.y1' ], data[ ,'rect.x2' ], data[ ,'rect.y2' ], col='lightgray' )
}

xmap.gene.plot = function( genes, style=c( 'real', 'stacked', 'jointed', 'injointed' ), rect.func=.default.plotter ) {
	style = match.arg( style )
	genes = xmapcore:::.get.correct.column( 'gene', genes )
	opar = par( mfrow=c(length(genes),1), mar=c(1,1,2,2), oma=c(0,0,0,0) )
	sapply( genes, function( gene.name ) {
				transcripts = gene.to.transcript( gene.name )
				gd = gene.details( gene.name )
				gene.title = paste( gene.name, if( gd$strand > 0 ) '(+)' else '(-)', gd$symbol ) 

				# start drawing at the bottom, so rev the list to get them in the same draworder as X:Map
				all.exons = rev( sapply( transcripts, function( trans ) { transcript.to.exon( trans, as.vector=F ) } ) )
				
				# sort the exons by position (*.to.exon always returns exons in translation order ie; rtl on rev-strand)
				all.exons = sapply( all.exons, function( exon.list ) { exon.list[order( start( exon.list ) ),] } )
				
				# Cache the number of transcripts
				transcript.count = dim( transcripts )[1]
				
				# Get the names of the transcripts
				transcript.names = names( all.exons )
				
				if( style == 'stacked' ) {
					# go from 1 to the max number of exons in any transcript of this gene (the 0.5 fudge is to get the scale to centralise on an exon)
					.gs.init.plot.area( c( 0.5, max( sapply( all.exons, function( exon.list ) { dim( exon.list )[1] } ) ) - 0.5 ), names( all.exons ), main=gene.title )
					
					#for every transcript
					for( y in seq_along( all.exons ) ) {
						rect.data = data.frame()
						a.transcript = all.exons[[y]]
						starts = start( a.transcript )
						for( x in seq_along( starts ) ) {
							rect.data = rbind( rect.data,
												data.frame( gene=gene.name,
															transcript=transcript.names[y],
															exon=a.transcript[['stable_id']][x],
															rect.x1=x - 0.5,
															rect.y1=y - 0.5,
															rect.x2=x + 0.5,
															rect.y2=y + 0.5 ) )
						}
						rect.func( rect.data )
					}
				}
				else if( style == 'jointed' ) {
					.gs.init.plot.area( c( 0, max( sapply( all.exons, function( exon.list ) { sum( width( exon.list ) ) } ) ) ), names( all.exons ), main=gene.title, xlab="Genetic distance" )
					for( y in seq_along( all.exons ) ) {
						r.offset = 0
						rect.data = data.frame()
						a.transcript = all.exons[[y]]
						widths = width( a.transcript )
						for( i in seq_along( widths ) ) {
							w = widths[ i ]
							rect.data = rbind( rect.data, 
												data.frame( gene=gene.name, 
															transcript=transcript.names[y], 
															exon=a.transcript[['stable_id']][i],
															rect.x1=r.offset,
															rect.y1=y - 0.5,
															rect.x2=r.offset + w,
															rect.y2=y + 0.5 ) )
							r.offset = r.offset + w
						}
						rect.func( rect.data )
					}
				}
				else if( style == 'injointed' ) {
					.gs.init.plot.area( c( 0, max( sapply( all.exons, function( exon.list ) { max( end( exon.list ) ) - min( start( exon.list ) ) } ) ) ), names( all.exons ), main=gene.title, xlab="Genetic distance" )
					for( y in seq_along( all.exons ) ) {
						r.offset = 0
						rect.data = data.frame()
						line.data = list( x=c(), y=c() )
						a.transcript = all.exons[[y]]
						num.exons = dim( a.transcript )[1]
						starts = start( a.transcript )
						ends = end( a.transcript )
						widths = width( a.transcript )
						for( i in seq_along( widths ) ) {
							w = widths[i]
							rect.data = rbind( rect.data,
												data.frame( gene=gene.name,
															transcript=transcript.names[y],
															exon=a.transcript[['stable_id']][i],
															rect.x1=r.offset,
															rect.y1=y - 0.5,
															rect.x2=r.offset + w,
															rect.y2=y + 0.5 ) )
							r.offset = r.offset + w
							if( i < num.exons ) {
								intron.length = starts[ i+1 ] - ends[ i ]
								line.data$x = c( line.data$x, r.offset, r.offset + intron.length, NA )
								line.data$y = c( line.data$y, y, y, NA )
								r.offset = r.offset + intron.length
							}
						}
						rect.func( rect.data )
						lines( line.data$x, line.data$y, lty=3 )
					}
				}
				else if( style == 'real' ) {
					.gs.init.plot.area( c( start( gd ), end( gd ) ), names( all.exons ), main=gene.title, xlab=paste( "Genetic location (chr ", space( gd ), ")", sep="" ) )
					for( y in seq_along( all.exons ) ) {
						rect.data = data.frame()
						line.data = list( x=c(), y=c() )
						a.transcript = all.exons[[y]]
						num.exons = dim( a.transcript )[1]
						starts = start( a.transcript )
						ends = end( a.transcript )
						for( i in seq_along( starts ) ) {
							a.exon = a.transcript[i,]
							rect.data = rbind( rect.data,
												data.frame( gene=gene.name,
															transcript=transcript.names[y],
															exon=a.transcript[['stable_id']][i],
															rect.x1=starts[i],
															rect.y1=y - 0.5,
															rect.x2=ends[i],
															rect.y2=y + 0.5 ) )
							if( i < num.exons ) {
								line.data$x = c( line.data$x, ends[i], starts[i+1], NA )
								line.data$y = c( line.data$y, y, y, NA )
							}
						}
						rect.func( rect.data )
						lines( line.data$x, line.data$y, lty=3 )
					}
				}
			})
	par( opar )
}