.get.user.string = function(v,msg) {
	if( v != "<ASK>" ) {
		r = v
	}
	else {
		cat( paste( msg, ":", sep="" ) )
		r = readLines( n=1 )
	}
	r
}

.xmc.all = function( src, params, dest=NULL ) {
	cache.id = paste( src, params$array, sep=':' )
	ret = .cache.retrieve( cache.id )
	if( is.null( ret ) ) {
		sql = .build.sql2( .xmap.queries[[ paste( src, "all", sep="." ) ]], params )
		ret = .xmc.db.call( sql, .xmap.internals$con )
		.cache.store( cache.id, ret )
	}
	ret
}

.xmc.details = function( src, params, dest=NULL ) {
	if( is.null( params ) ) {
		return( NULL )
	}
	sql = .build.sql2( .xmap.queries[[ paste( src, "details", sep="." ) ]], params )
	.xmc.db.call( sql, .xmap.internals$con )
}

.xmc.to = function( src, params, dest=NULL ) {
	if( is.null( params ) ) {
		return( NULL )
	}
	sql = .build.sql2( .xmap.queries[[ paste( src, "to", dest, sep="." ) ]], params )
	.xmc.db.call( sql, .xmap.internals$con )
}

.xmc.range = function( src, params, dest=NULL ) {
	sql = .build.sql2( .xmap.queries[[ paste( src, "range", sep="." ) ]], params )
	.xmc.db.call( sql, .xmap.internals$con )
}

.needs.array = function( action, src, dest=NULL ) {
	if( ( action == "all" || action == "details" ) && src == "array" ) {
		FALSE
	}
	else {
		sd = c( src, dest )
		any( sd == "probe" ) || any( sd == "array" ) || any( sd == "probeset" ) || any( sd == "cdnaprobeset" ) || any( sd == "exon_probeset" )
	}
}

.make.params = function( ids ) {
	if( is.null( .xmap.internals$connected ) || !.xmap.internals$connected ) {
	  stop( "You need to connect to a database. -- see xmap.connect()")
	}
	if( is.null( ids ) ) {
		stop( "In .make.params, ids is NULL" )
	}
	if( is.list( ids ) ) {
		ids = unlist( ids, use.names=FALSE )
	}
	one       = max( nchar( ids ) ) + 1
	max.list  = .xmap.internals$max.query %/% one
	if( max.list == 0 ) max.list = 1
	n.lists   = length( ids )  %/% max.list
	by.mat    = ( max.list * n.lists )
	.xmap.internals$debugFn( c( "one =", one, " - max.list =", max.list, " - n.lists =", n.lists, " - by.mat =", by.mat ) )
	if( n.lists == 0 ) {
		return( paste( sapply( ids, function( b ) { dbEscapeStrings( .xmap.internals$con, b ) } ), sep=",", collapse="," ) )
	}
	else {
		.xmap.internals$debugFn( c( "Splitting into", n.lists, "queries with", max.list, "entries per query" ) )
		mat = matrix( ids[ 1:by.mat ], nrow=n.lists, byrow=TRUE )
		.xmap.internals$debugFn( c( "matrix dimensions:", dim( mat ) ) )
		r   = split( mat, row( mat ) )
		.xmap.internals$debugFn( c( "length of r is", length( r ) ) )
		if( by.mat < length( ids ) ) {
			r = c( r, list( ids[ ( by.mat + 1 ):length( ids ) ] ) )
		}
		escaped = lapply( r, function( b ) { sapply( b, function( str ) { dbEscapeStrings( .xmap.internals$con, str ) } ) } )
		return( lapply( escaped, function( row ) { paste( row, sep=',', collapse=',' ) } ) )
	}
}

.duplicate.hash = function( oldEnv ) {
	ret = new.env( hash=TRUE )
	names = ls( envir=oldEnv )
	sapply( names, function( x ) { assign( x, get( x, envir=oldEnv ), envir=ret ) } )
	ret
}

.coerce = function( data, column, as.vector=TRUE ) {
	if( is.null( data ) ) {
		NULL
	}
	else if( as.vector == TRUE ) {
		unique( data[,column] )
	}
	else if( as.vector != 'data.frame' ) {
		data = subset( data, select=!( colnames( data ) %in% .xmap.internals$field.mask ) )
		if( is.null( column ) && !is.null( data ) ) {
			data$IN1 = NULL
		}
		if( any( colnames( data ) == "strand" ) ) {
			colnames( data )[ colnames( data ) == "chromosome_name" ] = "space"
			as( data, "RangedData" )
		}
		else {
			data
		}
	}
	else {
		data
	}
}

.get.correct.column = function( type, data ) {
	if( class( data ) == 'RangedData' || is.data.frame( data ) ) {
		.xmap.internals$debugFn( c( "Trying to select column", .xmap.types[[ type ]], "from id data.frame" ) )
		newIds = data[[ .xmap.types[[ type ]] ]]
		if( !is.null( newIds ) ) {
			data = newIds
		}
	}
	else if( is.list( data ) ) {
		.xmap.internals$debugFn( c( "Unlisting ids list of dim", dim( data ), "and length", length( data ) ) )
		newIds = unlist( data, use.names=FALSE )
		if( !is.null( newIds ) ) {
			data = newIds
		}
	}
	data
}

.process = function( action, src, params=NULL, dest=NULL ) {
	if( is.null( .xmap.internals$connected ) || !.xmap.internals$connected ) {
	  stop( "You need to connect to a database. -- see xmap.connect()")
	}
	# Check we have an array set if we need one...
	if( .needs.array( action, src, dest ) && is.null( .xmap.internals$array ) ) {
	  stop( "You need to set an array type. -- see xmap.array.type()")
	}
	else if( .needs.array( action, src, dest ) ) {
		if( is.null( params ) ) {
			params = new.env( hash=TRUE )
		}
		params$array = .xmap.internals$array
	}

	# Ok, do the ids in .xmap.internals$batch.size chunks
	fname = paste( ".xmc", action, sep="." )
	.xmap.internals$debugFn( c( "Calling", fname, "[", src, ":", dest, "]" ) )

	f = .xmap.internals$procs[[ action ]]
    
	if( !is.null( params ) && !is.null( params$ids ) ) {
		params$ids = .get.correct.column( src, params$ids )
		p = .duplicate.hash( params )
		r = lapply( .xmap.internals$procs$params( params$ids ), 
			function( x ) {
				p$ids = x
				f( src, p, dest )
			}
		)
		if( !is.null( r[[ 1 ]] ) ) {
			r = do.call("rbind",r)
			rownames(r) = 1:(dim(r)[1])
			r
		}
		else {
			NULL
		}
	}
	else {
		f( src, params, dest )
    }
}

gene.to.symbol = function( ids ) {
	if( is.vector( ids ) ) {
		ids = gene.details( ids )
	}
	ids[[ 'symbol' ]]
}

all.symbols = function( as.vector=TRUE ) {
	r = sort( unique( all.genes( as.vector=F )$symbol ) )
	if( as.vector == FALSE ) {
		r = data.frame( symbol=r )
	}
	r
}

xmap.range.apply = function( x, f, filter=c( chr="space", start="start", end="end", strand="strand" ),
		coerce=c( as.character, as.numeric, as.numeric, as.numeric ), ... ) {
	if( class( x ) != 'RangedData' ) {
		stop( paste( 'x must be a RangedData object, not a', class( x ) ) )
	}
	if( length( coerce ) != length( filter ) ) {
		stop( paste( "Each filter column should have a corresponding coercion function: coerce:length(", length(coerce), ") filter:length(", length( filter ), ")." ) )
	}
	framed.data             = as( x, "DataFrame" )
	framed.data             = framed.data[,filter,drop=FALSE]
	colnames( framed.data ) = names( filter )
	r = lapply( seq_len( dim( framed.data )[1] ), function( i ) {
		current.row = as.vector( framed.data[i,,drop=TRUE] )
                #Attempt to coerce the current row to the types defined for each parameter to 'f'
		current.row = lapply( seq_along( current.row ), function( a ) { do.call( coerce[[a]], list( current.row[[a]] ) ) } )
		params      = as.list( c( current.row, ... ) )
		do.call( f, params )
	} )
	# Strip nulls
	r = r[ !sapply( r, is.null ) ]
	clzz = unique( lapply( r, class ) )
	if( length( clzz ) == 2 && is.list( r ) ) {
		r = do.call( c, r )
		clzz = unique( lapply( r, class ) )
	}
	if( length( clzz ) == 1 ) {
		if( is.vector( r[[1]] ) ) {
			r = do.call( c, r )
		}
		else {
			r = do.call( rbind, r )
		}
	}
	r
}

gene.to.exon.probeset.expr = function( x, ids, probes.min=4 ) {
	if( length( ids ) == 0 ) {
		return( NULL )
	}
	if( class( x ) == 'ExpressionSet' ) {
		x = exprs( x )
	}
	pse  = gene.to.exon_probeset( ids, probes.min=probes.min )
	ps.r = pse[ , 'probeset' ]
	ps   = intersect( ps.r, rownames( x ) )
	r    = pse[ ps.r %in% ps, , drop=FALSE ]
	x    = as.data.frame( x[ rownames( x ) %in% ps, , drop=FALSE ] )
	x    = cbind( probeset=rownames( x ), x )
	r    = merge( x, r, by='probeset' )
	idx  = c( ( 2:length( colnames( x ) ) ), 1, ( dim( x )[ 2 ] + 1 ):length( r ) )
	r    = r[ , idx, drop=FALSE ]
	r
}

xmap.array.type = function( name=NULL, pick.default=FALSE ) {
	# Get a list of arrays from the database
	arrs = all.arrays( as.vector=FALSE )
	if( !is.null( name ) ) {
		# Check were not looking for a garbage name
		if( dim( arrs[ arrs[,1] == name, ] )[ 1 ] == 0 ) {
			stop( paste( "Array '", name, "' not found.  Call with no parameters to choose from a list", sep="" ) )
		}
	}
    if( is.null( name ) ) {
    	if( pick.default ) {
    		name = arrs[1,1]
    		cat( paste( "Selected array '", name, "' as a default.\n", sep="" ) )
        }
        else {
			choices = paste( as.character( arrs[,1] ), " ('", as.character( arrs[,2] ), "')", sep="" )
			r = menu( choices, title="Select an array type to use:" )
			if( r == 0 ) {
				return( invisible() )
			}
			else {
				name = as.character( arrs[r,1] )
			}
        }
    }
    .xmap.internals$array = name
    invisible()
}

transcript.to.translatedprobes = function( ids ) {
    transcript.ids = .get.correct.column( 'transcript', ids )
    if( is.null( transcript.ids ) ) {
    	return( NULL )
    }
    # Get all the exons for this transcript
    exons = transcript.to.exon( transcript.ids, as.vector=F )
    sapply( split( exons[['stable_id']], exons[['IN1']] ), function( exons ) {
        exons = exon.details( exons )
        fwd.strand = all( exons$strand == 1 )
        translated.width = sum( width( exons ) )
        # Go along each exon
        probes = sapply( seq_along( exons$stable_id ), function( eidx ) {
            # Get the details for this exon
            exon = exons[ eidx, ]

            # Find the probes which hit the same region as the exon
            probes = probes.in.range( as.character( space( exon ) ),
                                      start( exon ),
                                      end( exon ),
                                      exon$strand,
                                      as.vector=F )

            if( !is.null( probes ) ) {
                # Remove those that hang off the ends of the exon
                probes = probes[ start( probes ) >= start( exon ), ]
                probes = probes[ end( probes ) <= end( exon ), ]
            }

            # Return the probes for this exon
            probes
        } )

        # Then, change these offsets so that they are relative to the start of the
        # transcript
        exon.offset = 0
        for( idx in seq_along( exons$stable_id ) ) {
            # For the reverse strand, go through the list backwards
            eidx = if( fwd.strand ) idx else length( exons$stable_id ) - idx + 1
            if( !is.null( probes[[ eidx ]] ) ) {
                ranges( probes[[ eidx ]] ) = shift( ranges( probes[[ eidx ]] ),
                                                   -start( exons[ eidx, ] ) + exon.offset )
                # If on the reverse strand, change offsets to 5' from 3'
                if( !fwd.strand ) {
                    ranges( probes[[ eidx ]] ) = IRanges( start=translated.width - end( probes[[ eidx ]] ),
                                                          end=translated.width - start( probes[[ eidx ]] ) )
                    # Re-sort the probes by start
                    probes[[ eidx ]] = probes[[ eidx ]][ order( start( probes[[ eidx ]] ) ), ]
                }
            }
            exon.offset = exon.offset + width( exons[ eidx, ] )
        }

        # Remove all NULLs and combine
        probes = do.call( 'rbind', probes[ !sapply( probes, is.null ) ] )
    } )
}

.set.conf.dir = function() {
	tryCatch( rm( conf.dir, envir=.xmap.internals ), warning=function(a){invisible()} )
	conf.dir = Sys.getenv( "XMAPCORE_HOME" )
	if( conf.dir == "" ) { 
		conf.dir = file.path( Sys.getenv( "HOME" ), ".xmapcore" )
    }
    cat( paste( "Using", conf.dir, "as our configuration directory\n" ) )
	if( !file.exists( conf.dir ) ) {
		warning( paste( "Folder", conf.dir, "does not exist.  Attempting to create." ) )
		dir.create( conf.dir, recursive=TRUE )
		if( !file.exists( conf.dir ) ) {
			stop( paste( "Failed to create folder '", conf.dir, "', giving up.", sep="" ) )
		}
	}
	.xmap.internals$conf.dir = conf.dir
}

xmap.set.param = function( ... ) {
	.params = list( ... )
	for( .name in names( .params ) ) {
		.xmap.internals[[ .name ]] = .params[ .name ][[1]]
	}
	if( !is.null( .xmap.internals$debug ) && ( .xmap.internals$debug ) ) {
		.xmap.internals$debugFn = .debug.full
	}
	else {
		.xmap.internals$debugFn = .debug.none
	}
}

xmap.get.param = function( key ) {
	.xmap.internals[[ key ]]
}

.debug.none = function( message ) {
}

.debug.full = function( message ) {
	cat( format( Sys.time(), "%a %b %d %X %Y" ), "::", message, "\n" )
}

.initialise = function( use.cache=TRUE ) {
	.set.conf.dir()
	.xmap.internals$debug = FALSE
	.xmap.internals$debugFn = .debug.none
	.xmap.internals$max.query = 10000
	.xmap.internals$connected = FALSE
	.xmap.internals$field.mask = c( 'array_id', 'chromosome_id', 'hit_id', 'protein_id', 'gene_id', 'transcript_id', 'exon_id', 'probe_id', 'probeset_id', 'synonym_id', 'external_db_id' )
	
	# Put all our main calls in an internal hash
	# We should then be able to call xmap.set.param with a new hash
	# to override these (eg for using a webservice rather than a db)
	.procs = .make.hash()
	.procs$all = .xmc.all
	.procs$details = .xmc.details
	.procs$to = .xmc.to
	.procs$range = .xmc.range
	.procs$params = .make.params
	.procs$connect = .xmc.connect
	.procs$disconnect = .xmc.disconnect
	.xmap.internals$procs = .procs
	.xmap.internals$initialised = TRUE

	# Set the cache root
	path = file.path( .xmap.internals$conf.dir, "cache" )
	if( !file.exists( path ) ) {
		r = dir.create( path )
		if( !r ) {
			warning( paste( "Could not create or locate cache folder '", path, "', running with caching disabled", sep="" ) )
			path = NULL
			use.cache = FALSE
		}
		else {
			use.cache = TRUE
		}
	}
	.xmap.internals$use.cache = use.cache
	.set.cache.root( path )
}

xmap.env = function() {
	if( is.null( .xmap.internals$connected ) || !.xmap.internals$connected ) {
	  stop( "You need to connect to a database. -- see xmap.connect()")
	}
	.xmc.db.call( "CALL env()", .xmap.internals$con )
}
