# This script builds the cookbook if the correct DB name is set up
# Or else does nothing, and leaves the PDF where it is...
#
# http://bit.ly/delia_smith

library( "xmapcore" )

data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
if( length( data ) == 1 ) {
	# The here's one I prepared earlier shortcut
	message( "Cannot find datasource 'hs-test', I'm not going to build the cookbook." )
} else {
	message( "Generating cookbook..." )
	# First, we'll disconnect, so the Rnw gets a fresh start
	xmap.disconnect()
	
	current.path = getwd()
	path <- file.path( getwd(), "..", "cookbook" )
	setwd( path )
	# Parse the Rnw
	library(tools)
	utils::Sweave( 'cookbook.Rnw' )

	# Generate the PDF
	texi2dvi( 'cookbook.tex', pdf=T, clean=T )

	# Copy it to the docs
	file.copy( 'cookbook.pdf', file.path( path, '..', 'doc' ), overwrite=T )

	# cleanup
	unlink( 'cookbook.pdf' )
	unlink( 'cookbook.tex' )
	unlink( 'fig1.pdf' )
	unlink( 'fig2.pdf' )
	unlink( 'fig3.pdf' )
	unlink( 'fig4.pdf' )
	unlink( 'fig5.pdf' )
	unlink( 'fig6.pdf' )

	message( "Generating Splicing Index Documentation" )
	utils::Sweave( 'SplicingIndexExample.Rnw' )
	# Generate the PDF
	texi2dvi( 'SplicingIndexExample.tex', pdf=T, clean=T )
	# Copy it to the docs
	file.copy( 'SplicingIndexExample.pdf', file.path( path, '..', 'doc' ), overwrite=T )
	# cleanup
	unlink( 'SplicingIndexExample.pdf' )
	unlink( 'SplicingIndexExample.tex' )

	setwd( current.path )
}

