# Testing for the db functions of the package

if(FALSE) {
	library( "RUnit" )
	library( "xmapcore" )
}

###############################################################################
## Create a test environment
##
.test.env = new.env( hash=TRUE, parent=emptyenv() )

###############################################################################
## SetUp is called before each test method
##
.setUp = function() {
}

###############################################################################
## TearDown is called after each test method
##
.tearDown = function() {
}

test.databases.txt = function() {
	if( Sys.getenv( "RCMDCHECK" ) == "FALSE" ) {
		path = file.path( getwd(), "..", "inst", "unitTests" )
	}
	else {
		path = system.file( package=pkg, "unitTests" )
	}
	old.dir = xmap.get.param( 'conf.dir' )
  .procs = xmap.get.param( 'procs' )
	print( 'Mocking xmap.connect()' )
	.procs$connect = function( name ) {
		f = xmapcore:::.xmap.internals$conf.dir
		f = file.path( f, "databases.txt" )
		if( !file.exists( f ) ) {
			stop( paste( "Can't find database config file '", f, "'. See package installation instructions for help setting this up.", sep="" ) )
		}
		a = read.table( f, sep="\t", strip.white=TRUE, header=TRUE )
		if( !all( colnames( a ) == c( "name", "host", "species", "version", "port", "username", "password" ) ) ) {
			a = read.table( f, sep=",", strip.white=TRUE, header=TRUE )
		}
		if( !all( colnames( a ) == c( "name", "host", "species", "version", "port", "username", "password" ) ) ) {
			stop( paste( "Unexpected column names in",  f, " - should be 'name', 'host', 'species', 'version', 'port', 'username' and 'password'" ) )
		}
		dbs = a[ ( as.character( a$name ) == name ), , drop=FALSE ]
		if( dim( dbs )[1] == 0 ) { stop( paste( "Invalid db name:", name ) ) }
		as.character( dbs[,"host"] )
	}
  xmap.set.param( procs=.procs )
	print( paste( 'setting conf.dir to', file.path( path, 'tabConfig' ) ) )
	xmap.set.param( conf.dir = file.path( path, 'tabConfig' ) )
	checkEquals( xmap.connect( 'testdb' ), 'localhost', 'Should connect to localhost with tab separated config' )
	print( paste( 'setting conf.dir to', file.path( path, 'commaConfig' ) ) )
	xmap.set.param( conf.dir = file.path( path, 'commaConfig' ) )
	checkEquals( xmap.connect( 'testdb' ), 'localhost', 'Should connect to localhost with comma separated config' )
	print( paste( 'setting conf.dir back to', old.dir ) )
	xmap.set.param( conf.dir = old.dir )

	print( 'restoring connect method' )
  .procs$connect = xmapcore:::.xmc.connect
  xmap.set.param( procs=.procs )
}

test.connection = function() {
	checkEquals( xmap.get.param( "connected" ), FALSE, "Start not connected to anything" )
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		checkTrue( xmap.get.param( "connected" ), "Should now be connected" )
		xmap.disconnect()
		checkTrue( !xmap.get.param( "connected" ), "And connected status should be FALSE" )
	}
}

test.buildsql = function() {
	test.hash = new.env( hash=TRUE )
	test.hash$a = "p1"
	test.hash$b = "p2"
	test.hash$c = "p3"
	sql = xmapcore:::.build.sql2( "Parameter replacement for ${a}, ${b} and ${c} should be ok", test.hash )
	checkEquals( sql, paste( "Parameter replacement for ", test.hash$a, ", ", test.hash$b, " and ", test.hash$c, " should be ok", sep="" ) )
}
