# Testing all of the statements (to make sure none of them crash)
# Testing for acutal responses is more tricky, as the responses may change from one
# release to the next...

if(FALSE) {
  library( "RUnit" )
  library( "xmapcore" )
}

###############################################################################
## Utility methods
##
.get.functions = function( func.type, func.pattern, min.len, name.build.function ) {
	.functions = ls( envir=xmapcore:::.xmap.queries )
	.queries = strsplit( .functions[ grep( func.pattern, .functions ) ], '\\.' )
	checkTrue( length( .queries ) >= min.len, paste( "Should have at least ", min.len, " '", func.type, "' functions", sep="" ) )
	r = lapply( .queries, function(i) {
		name = name.build.function( i )
		c( name, getFunction( name ) )
	} )
	r
}

.run.full.test.suite = function() {
	1 == length( grep( 'picr.man.ac.uk', Sys.info()[ 'nodename' ] ) )
}

.run.functions = function( functions, executor ) {
	lapply( c( 'hs-test', 'mm-test', 'rn-test', 'pb-test' ), function( db ) {
		data = tryCatch( xmap.connect( db ), error=function(e) { FALSE } )
		if( length( data ) == 1 ) {
			print( paste( "Cannot find datasource '", db, "', so skipping these tests.", sep="" ) )
		}
		else {
			# Set this to true if this species has no arrays
			no.arrays = is.null( all.arrays( as.vector=F ) )
			lapply( functions, function( func ) {
				if( no.arrays && length( grep( "array|probe|hit|est_|prediction", func[[1]] ) ) == 1 ) {
					print( paste( "Skipping", func[[1]], "on", xmapcore:::.xmap.internals$db.name ) )
				}
				else {
					print( paste( "Running", func[[1]], "on", xmapcore:::.xmap.internals$db.name ) )
					tryCatch( executor( func[[1]], func[[2]] ), error=function(e) {
						traceback()
						print( e )
					} )
				}
			} )
		}
	} )
}

###############################################################################
## SetUp is called before each test method
##
.setUp = function() {
}

###############################################################################
## TearDown is called after each test method
##
.tearDown = function() {
	xmap.disconnect()
}

test.gotsomefunctions = function() {
  checkTrue( length( ls( envir=xmapcore:::.xmap.queries ) ) >= 96, "Should have at least 96 functions" )
}

test.transcript.to.translatedprobes = function() {
	data = tryCatch( xmap.connect( 'hs-test' ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( paste( "Cannot find datasource 'hs-test', so skipping these tests.", sep="" ) )
	}
	else {
		transcripts = gene.to.transcript( symbol.to.gene( 'TP53' ) )
		checkTrue( length( transcript.to.translatedprobes( transcripts ) ) == length( transcripts ), "vector transcript.to.translatedprobes lengths differ" )
		transcripts = gene.to.transcript( symbol.to.gene( 'TP53' ), as.vector=FALSE )
		checkTrue( length( transcript.to.translatedprobes( transcripts ) ) == dim( transcripts )[1], "RangedData transcript.to.translatedprobes lengths differ" )
	}
}

test.to.queries = function() {
	# Get all 'to' functions (of which there should be at least 58)
	.functions = .get.functions( 'to', '\\.to\\.', 58, function(i) { paste( i[1], '.to.', i[3], sep="" ) } )

	# And run them, searching for the thing called 'InvalidId' which should return no results, but not crash)
    .run.functions( .functions,  function( name, f ) {
		if( !.run.full.test.suite() && name == "array.to.probeset" ) {
			print( '\nSkipping array.to.probeset test, as not on a PICR machine' )
		}
		else {
			r = f( 'InvalidId' )
			if( name == "array.to.probeset" ) {
				checkTrue( length( r ) > 0, "should have some probesets for an array" )
			}
			else {
				checkTrue( is.null( r ), "should return null" )
			}
		}
	} )
}

test.expr.query = function() {
	data = tryCatch( xmap.connect( 'hs-test' ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( paste( "Cannot find datasource 'hs-test', so skipping these tests.", sep="" ) )
	}
	else {
		if( Sys.getenv( "RCMDCHECK" ) == "FALSE" ) {
			path = file.path( getwd(), "..", "inst", "unitTests" )
		}
		else {
			path = system.file( package=pkg, "unitTests" )
		}
		lapply( all.arrays(), function( arr ) {
			xmap.array.type( arr )
			file.name = file.path( path, paste( arr, 'tp53.expr.RData', sep='.' ) )
			if( file.exists( file.name ) ) {
				env = new.env()
				print( paste( 'Loading', file.name ) )
				load( file.name, env )
				o.name = ls( env )[ 1 ]
				print( paste( 'Using variable', o.name ) )
				rslt = gene.to.exon.probeset.expr( get( o.name, envir=env ), symbol.to.gene( 'tp53' ) )
				rslt.dim = dim( rslt )
				checkTrue( rslt.dim[ 1 ] > 0, 'Should have rows in out exprs result data' )
				checkTrue( rslt.dim[ 2 ] > 0, 'Should have columns in out exprs result data' )
			}
			else {
				print( paste( '***WARN*** Cannot find file', file.name, 'to test gene.to.exon.probeset.expr for', arr ) )
			}
		} )
	}
}
