# Testing for the utility functions of the package

if(FALSE) {
	library( "RUnit" )
	library( "xmapcore" )
}

###############################################################################
## SetUp is called before each test method
##
.setUp = function() {
}

###############################################################################
## TearDown is called after each test method
##
.tearDown = function() {
}

test.utr = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		gene = symbol.to.gene( "tp53" )
	    probesets.vector = gene.to.probeset( gene )
	    probesets.frame  = gene.to.probeset( gene, as.vector=FALSE )
		
		both.vector = utr.probesets( probesets.vector )
		both.frame  = utr.probesets( probesets.frame )
		checkEquals( length( both.vector ), length( both.frame ), "Should get the same results with Vector and Frame (both UTRs)" )
		
		five.vector = utr.probesets( probesets.vector, end="5" )
		five.frame  = utr.probesets( probesets.frame,  end="5" )
		checkEquals( length( five.vector ), length( five.frame ), "Should get the same results with Vector and Frame (5' end)" )
		
		transcripts = gene.to.transcript( gene )
		singletranscript.vector = utr.probesets( probesets.vector, c( transcripts[1] ) )
		singletranscript.frame  = utr.probesets( probesets.frame,  c( transcripts[1] ) )
		checkEquals( length( singletranscript.vector ), length( singletranscript.frame ), "Should get the same results with Vector and Frame on a single Transcript (both UTRs)" )
		
		xmap.disconnect()
	}
}

test.yaoyong.utr.bug = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		# Should not get any intronic probesets back for this transcript
		yy.probesets = utr.probesets( transcript.to.probeset( 'ENST00000378357' ), 'ENST00000378357' )
		checkEquals( length( intronic( yy.probesets ) ), 0 )
	}
}

test.michal.utr.bug = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		# These hit exons inside the UTR exons (not just the exon where translation starts)
		checkEquals( utr.probesets( '3855309', 'ENST00000247005' ), '3855309' )
		checkEquals( utr.probesets( '3855308', 'ENST00000247005' ), '3855308' )
		checkEquals( utr.probesets( '3855297', 'ENST00000247005' ), '3855297' )
		checkEquals( utr.probesets( '3855316', 'ENST00000247005' ), '3855316' )
		# This probeset is inside the translated region
		checkTrue( is.null( utr.probesets( '3855295', 'ENST00000247005' ) ) )
	}
}

test.michal.utr.bug.two = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		g   = 'ENSG00000130283'
		ppp = gene.to.probeset( g )
		ttt = gene.to.transcript( g )
		checkEquals( length( utr.probesets( ppp, ttt ) ), 27 )
	}
}

# see bug XMAPCORE-3
test.michal.utr.bug.three = function() {
	data = tryCatch( xmap.connect( "hs-test" ), error=function(e) { FALSE } )
	if( length( data ) == 1 ) {
		print( "Cannot find datasource 'hs-test', so skipping these tests." )
	}
	else {
		pp.t = exonic( transcript.to.probeset( 'ENST00000455250' ) )
		checkTrue( length( pp.t ) > 0 )
		checkTrue( is.null( utr.probesets( pp.t, "ENST00000455250" ) ) )
	}
}

